@@ -3,7 +3,7 @@
 import re
 import sys
 import json
-from datetime import datetime
+from datetime import datetime, timedelta
 from waybackpy.exceptions import WaybackError
 from waybackpy.__version__ import __version__
 
@@ -69,21 +69,77 @@ class Url:
         self.url = url
         self.user_agent = user_agent
         self._url_check()  # checks url validity on init.
+        self.JSON = self._JSON() # JSON of most recent archive
+        self.archive_url = self._archive_url() # URL of archive
+        self.timestamp = self._archive_timestamp() # timestamp for last archive
 
     def __repr__(self):
         return "waybackpy.Url(url=%s, user_agent=%s)" % (self.url, self.user_agent)
 
     def __str__(self):
-        return "%s" % self._clean_url()
+        return "%s" % self.archive_url
 
     def __len__(self):
-        return len(self._clean_url())
+        td_max = timedelta(days=999999999,
+                             hours=23,
+                             minutes=59,
+                             seconds=59,
+                             microseconds=999999)
+        if self.timestamp == datetime.max:
+            return td_max.days
+        else:
+            diff = datetime.utcnow() - self.timestamp
+            return diff.days
 
     def _url_check(self):
         """Check for common URL problems."""
         if "." not in self.url:
             raise URLError("'%s' is not a vaild URL." % self.url)
 
+    def _JSON(self):
+        request_url = "https://archive.org/wayback/available?url=%s" % (
+            self._clean_url(),
+        )
+
+        hdr = {"User-Agent": "%s" % self.user_agent}
+        req = Request(request_url, headers=hdr)  # nosec
+        response = _get_response(req)
+        data_string = response.read().decode("UTF-8")
+        data = json.loads(data_string)
+
+        return data
+
+    def _archive_url(self):
+        """Get URL of archive."""
+        data = self.JSON
+
+        if not data["archived_snapshots"]:
+            archive_url = None
+        else:
+            archive_url = data["archived_snapshots"]["closest"]["url"]
+            archive_url = archive_url.replace(
+                "http://web.archive.org/web/",
+                "https://web.archive.org/web/",
+                1
+            )
+
+        return archive_url
+
+    def _archive_timestamp(self):
+        """Get timestamp of last archive."""
+        data = self.JSON
+
+        if not data["archived_snapshots"]:
+            time = datetime.max
+
+        else:
+            time = datetime.strptime(data["archived_snapshots"]
+                                     ["closest"]
+                                     ["timestamp"],
+                                     '%Y%m%d%H%M%S')
+
+        return time
+
     def _clean_url(self):
         """Fix the URL, if possible."""
         return str(self.url).strip().replace(" ", "_")
@@ -94,14 +150,18 @@ class Url:
         hdr = {"User-Agent": "%s" % self.user_agent}  # nosec
         req = Request(request_url, headers=hdr)  # nosec
         header = _get_response(req).headers
-        return "https://" + _archive_url_parser(header)
+        self.archive_url = "https://" + _archive_url_parser(header)
+        self.timestamp = datetime.utcnow()
+        return self
 
     def get(self, url="", user_agent="", encoding=""):
         """Return the source code of the supplied URL.
         If encoding is not supplied, it is auto-detected from the response.
         """
+
         if not url:
             url = self._clean_url()
+
         if not user_agent:
             user_agent = self.user_agent
 
@@ -144,11 +204,18 @@ class Url:
                 "to create a new archive." % self._clean_url()
             )
         archive_url = data["archived_snapshots"]["closest"]["url"]
-        # wayback machine returns http sometimes, idk why? But they support https
         archive_url = archive_url.replace(
             "http://web.archive.org/web/", "https://web.archive.org/web/", 1
         )
-        return archive_url
+
+        self.archive_url = archive_url
+        self.timestamp = datetime.strptime(data["archived_snapshots"]
+                                 ["closest"]
+                                 ["timestamp"],
+                                 '%Y%m%d%H%M%S')
+
+        return self
+
 
     def oldest(self, year=1994):
         """Return the oldest Wayback Machine archive for this URL."""
@@ -173,3 +240,50 @@ class Url:
         response = _get_response(req)
         # Most efficient method to count number of archives (yet)
         return str(response.read()).count(",")
+
+    def known_urls(self, alive=False, subdomain=False):
+        """Returns list of URLs known to exist for given domain name
+        because these URLs were crawled by WayBack Machine bots.
+
+        Useful for pen-testers and others.
+
+        Idea by Mohammed Diaa (https://github.com/mhmdiaa) from:
+        https://gist.github.com/mhmdiaa/adf6bff70142e5091792841d4b372050
+        """
+
+        url_list = []
+
+        if subdomain:
+            request_url = (
+            "https://web.archive.org/cdx/search/cdx?url=*.%s/*&output=json&fl=original&collapse=urlkey"
+            % self._clean_url()
+            )
+
+        else:
+            request_url = (
+            "http://web.archive.org/cdx/search/cdx?url=%s/*&output=json&fl=original&collapse=urlkey"
+            % self._clean_url()
+            )
+
+        hdr = {"User-Agent": "%s" % self.user_agent}
+        req = Request(request_url, headers=hdr)  # nosec
+        response = _get_response(req)
+
+        data = json.loads(response.read().decode("UTF-8"))
+        url_list = [y[0] for y in data if y[0] != "original"]
+
+        #Remove all deadURLs from url_list if alive=True
+        if alive:
+            tmp_url_list = []
+            for url in url_list:
+
+                try:
+                    urlopen(url) # nosec
+                except:
+                    continue
+
+                tmp_url_list.append(url)
+
+            url_list = tmp_url_list
+
+        return url_list
